﻿/*!
Copyright (c) 2006, 2007, 2008, 2009, 2010 Data Illusion Zumbrunn. (http://www.dataillusion.com). All rights reserved.
Redistributions of source code must retain the above copyright notice and is licensed 
under the FeedbackServer End User License Agreement version provided with this package

*/
YAHOO.namespace ("DataIllusion.FeedbackServer");

YAHOO.DataIllusion.FeedbackServer.CSSHandler = function(styleId, validClasses, cssLoaded_callBack, styleCode)
{
	this.init(styleId, validClasses, cssLoaded_callBack, styleCode);
};

YAHOO.DataIllusion.FeedbackServer.CSSHandler.prototype =
{
	styleId: null,
	classElements: [],
	classStyles: [],
	classProperties: [],
	validClasses: Array,
	cssLoaded_callBack: null,

	init: function(styleId, validClasses, cssLoaded_callBack, styleCode)
	{
		this.styleId = styleId;
		this.validClasses = validClasses;
		this._initClassElements(document.childNodes);
		this.cssLoaded_callBack = cssLoaded_callBack;

		if (styleId && !styleCode) 
		{
			// Loads the styles from the DB
			DataIllusion.FeedbackServer.Services.Web.Admin.FormStylesService.GetCSSCode(this.styleId,
				this.loadStyle_callback, this.onFailed, this);
		}
		else if (styleCode) 
		{
			this.loadStyle(styleCode);
		}
	},

	// Assign the style to the page elements
	loadStyle_callback: function(res, styleHandler)
	{
		if (res) {
			styleHandler.loadStyle(res);

			if (styleHandler.cssLoaded_callBack) {
				styleHandler.cssLoaded_callBack();
			}

			styleHandler.applyStyleToDocument();
		}
	},

	// Show error message of the 
	// WS if something went wrong
	onFailed: function(error)
	{
		if (error !== null) {
			alert("An error occurred: " +
				error.get_message());
		}
	},

	// Apply the given style CSS to the page element
	// being FeedbackServer CSS classes
	loadStyle: function(styleCode)
	{
		// Remove all comments, new line chars
		styleCode = styleCode.replace(new RegExp("\\/\\*.*\\/|\\t|\\r\\n|\\n", "g"), "");

		var parseIndex = 0;

		while (parseIndex < styleCode.length) {
			classNameEndIndex = styleCode.indexOf('{', parseIndex),
			classStyleEndIndex = styleCode.indexOf('}', classNameEndIndex)

			if (classNameEndIndex > -1 && classStyleEndIndex > -1) {
				var className = styleCode.substr(parseIndex, classNameEndIndex - parseIndex),
					classStyle = styleCode.substr(classNameEndIndex + 1, classStyleEndIndex - classNameEndIndex - 1);

				// Is this a feedback sever class ?
				if (className.indexOf(".feedbackserver") > -1 && className.indexOf(" td") == -1) {
					className = className.replace(new RegExp("\\.feedbackserver|\\.|\ ", "g"), "");

					// Store the class properties and its style
					this._storeProperties(className.trim(), classStyle.trim());

					// Do we have already this class 
					// in our style collection ?
					if (!this.classStyles[className]) {
						this.classStyles[className] = [];
						this.classStyles[className][this.classStyles[className].length] = classStyle.trim();
					}
				}

				parseIndex = classStyleEndIndex + 1;
			}
			else {
				break;
			}
		}
	},

	// Returns the text ouptut of 
	// all the FeedbackServer style classes
	getStyle: function()
	{
		var style = "";

		// For each class take it style to 
		// build to global style
		for (var className in this.classStyles) {
			if (this.classStyles[className] && this.classStyles[className].length > 0) {
				style += ".feedbackserver ." + className + " {";
				style += this.classStyles[className];
				style += "}";
			}
		}

		return style;
	},


	// Apply all our class styles to 
	// our doucment elements
	applyStyleToDocument: function()
	{
		for (var className in this.classStyles) {
			if (this.classElements[className] && this.classElements[className].length > 0) {
				for (var i = 0; i < this.classElements[className].length; i++) {
					var styleProperties = this._getPropertiesFromStyle(this.classStyles[className]);

					// Reset the current element styles
					this.classElements[className][i].style.backgroundColor = '';
					this.classElements[className][i].style.fontFamily = '';
					this.classElements[className][i].style.fontSize = '';
					this.classElements[className][i].style.fontWeight = 100;
					this.classElements[className][i].style.borderWidth = '';
					this.classElements[className][i].style.borderStyle = '';
					this.classElements[className][i].style.width = '';
					this.classElements[className][i].style.padding = '';
					this.classElements[className][i].style.color = '';
					this.classElements[className][i].style.borderColor = '';

					// Apply the "live" element style values
					for (var j = 0; j < styleProperties.length; j++) {
						switch (styleProperties[j].styleName) {
							case "background-color":
								{
									this.classElements[className][i].style.backgroundColor = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "font-family":
								{
									this.classElements[className][i].style.fontFamily = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "font-size":
								{
									this.classElements[className][i].style.fontSize = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "font-weight":
								{
									// Do we have a valid weight value ?
									if (this.isUnsignedInteger(styleProperties[j].styleValue)) {
										// Assign that weight to the element. Note : IE supports only integer based values to set the weight
										this.classElements[className][i].style.fontWeight = parseInt(styleProperties[j].styleValue);
									}
									else {
										this.classElements[className][i].style.fontWeight = 100;
									}
									break;
								}
							case "border-width":
								{
									this.classElements[className][i].style.borderWidth = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "border-style":
								{
									this.classElements[className][i].style.borderStyle = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "width":
								{
									this.classElements[className][i].style.width = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "padding":
								{
									this.classElements[className][i].style.padding = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "color":
								{
									this.classElements[className][i].style.color = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}
							case "border-color":
								{
									this.classElements[className][i].style.borderColor = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
										styleProperties[j].styleValue : '';
									break;
								}

						}
					}
				}
			}
		}
	},

	isUnsignedInteger: function(value)
	{
		return (value.toString().search(/^[0-9]+$/) == 0);
	},

	// Apply the give style to the child element types of all
	// parent element for which this style has been applied to
	applyStyleToElementChilds: function(className, style, childTagName)
	{
		if (this.classElements[className] && this.classElements[className].length > 0) {
			for (var i = 0; i < this.classElements[className].length; i++) {
				this.applyStyleToChilds(this.classElements[className][i], style, childTagName);
			}
		}
	},

	// Apply the given style in the hiearchy on
	// all given element matching the tags 
	applyStyleToChilds: function(element, style, childTagName)
	{
		for (var i = 0; i < element.childNodes.length; i++) {
			if (element.childNodes[i].tagName && !element.childNodes[i].className &&
				element.childNodes[i].tagName.toUpperCase() == childTagName.toUpperCase()) {
				var styleProperties = this._getPropertiesFromStyle(style);

				// Reset the current element styles
				element.childNodes[i].style.backgroundColor = '';
				element.childNodes[i].style.fontFamily = '';
				element.childNodes[i].style.fontSize = '';
				element.childNodes[i].style.fontWeight = 100;
				element.childNodes[i].style.borderWidth = '';
				element.childNodes[i].style.borderStyle = '';
				element.childNodes[i].style.padding = '';
				element.childNodes[i].style.color = '';
				element.childNodes[i].style.borderColor = '';

				// Apply the "live" element style values
				for (var j = 0; j < styleProperties.length; j++) {
					switch (styleProperties[j].styleName) {
						case "background-color":
							{
								element.childNodes[i].style.backgroundColor = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "font-family":
							{
								element.childNodes[i].style.fontFamily = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "font-size":
							{
								element.childNodes[i].style.fontSize = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "font-weight":
							{
								// Do we have a valid weight value ?
								if (this.isUnsignedInteger(styleProperties[j].styleValue)) {
									// Assign that weight to the element. Note : IE supports only integer based values to set the weight
									element.childNodes[i].style.fontWeight = parseInt(styleProperties[j].styleValue);
								}
								else {
									element.childNodes[i].style.fontWeight = 100;
								}
								break;
							}
						case "border-width":
							{
								element.childNodes[i].style.borderWidth = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "border-style":
							{
								element.childNodes[i].style.borderStyle = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "width":
							{
								element.childNodes[i].style.width = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "padding":
							{
								element.childNodes[i].style.padding = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "color":
							{
								element.childNodes[i].style.color = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}
						case "border-color":
							{
								element.childNodes[i].style.borderColor = styleProperties[j].styleValue && styleProperties[j].styleValue.length > 0 ?
									styleProperties[j].styleValue : '';
								break;
							}

					}
				}
			}

			if (element.childNodes[i].tagName == "TABLE") {
				break;
			}

			this.applyStyleToChilds(element.childNodes[i], style, childTagName);
		}
	},

	// Assigns the given style to the given
	// class
	setClassStyle: function(className, style, applyStyle)
	{
		if (!this.classStyles[className]) {
			this.classStyles[className] = [];
		}

		// If class styles already exists update
		// it with our new style properties
		if (this.classStyles[className]) {
			this.classStyles[className] = style;
			if (this.classProperties[className]) {
				this.classProperties[className] = [];
			}

			this._storeProperties(className, style);
		}

		if (applyStyle) {
			// Apply the style to all document elements
			// being in that css class
			this.applyStyleToDocument();
		}
	},

	// Returns the property value of the given 
	// class and style property 
	getPropertieValue: function(className, styleName)
	{
		if (this.classProperties[className]) {
			for (var i = 0; i < this.classProperties[className].length; i++) {
				if (this.classProperties[className][i].styleName == styleName && this.classProperties[className][i].styleValue) {
					return this.classProperties[className][i].styleValue;
				}
			}
		}

		return '';
	},

	refreshClassElements: function()
	{
		classElements: [];
		this._initClassElements(document.childNodes);
	},

	// Loads all document elements
	// instances and keep them to be
	// able to assign styles to them
	_initClassElements: function(elements)
	{
		for (var i = 0; i < elements.length; i++) {
			// Check we can handle this kind of CSS class 
			if (elements[i].className != null && elements[i].className.length > 0 &&
				this._isClassValid(elements[i].className)) {
				if (!this.classElements[elements[i].className]) {
					this.classElements[elements[i].className] = [];
				}

				// Keep track of the element for this class
				this.classElements[elements[i].className]
					[this.classElements[elements[i].className].length] = elements[i];
			}

			this._initClassElements(elements[i].childNodes);
		}
	},

	// Check we can handle this kind of CSS class 
	_isClassValid: function(className)
	{
		for (var i = 0; i < this.validClasses.length; i++) {
			if (this.validClasses[i] == className) {
				return true;
			}
		}

		return false;
	},

	// Split all properties and save them
	// separately into an array to ease retrieval
	_storeProperties: function(className, style)
	{

		// Do we have already a set of properties
		// for this class ?
		if (!this.classProperties[className]) {
			this.classProperties[className] = [];
		}

		style = style.trim();

		var styles = style.split(';');
		for (var i = 0; i < styles.length - 1; i++) {
			styleName = styles[i].substr(0, styles[i].indexOf(':'));
			styleProperties = styles[i].substr(styles[i].indexOf(':') + 1);

			if (styleProperties && styleProperties.trim().length > 0) {
				// Add this new style property to our class
				this.classProperties[className][this.classProperties[className].length] = new 
					YAHOO.DataIllusion.FeedbackServer.CSSStyle(styleName.trim(),
					styleProperties.trim());
			}
		}
	},

	// Returns an array of all properties / values
	// available in the given style string
	_getPropertiesFromStyle: function(style)
	{
		style = style.toString().trim();
		var stylePropertiesCollection = [];

		var styles = style.split(';');
		for (var i = 0; i < styles.length - 1; i++) {
			styleName = styles[i].substr(0, styles[i].indexOf(':'));
			styleProperties = styles[i].substr(styles[i].indexOf(':') + 1);

			if (styleProperties && styleProperties.trim().length > 0) {
				// Add this new style property
				stylePropertiesCollection[stylePropertiesCollection.length] = new 
					YAHOO.DataIllusion.FeedbackServer.CSSStyle(styleName.trim(),
					styleProperties.trim());
			}
		}

		return stylePropertiesCollection;
	}
};



// CSS property and its value
YAHOO.DataIllusion.FeedbackServer.CSSStyle = function(styleName, styleValue)
{
	this.init(styleName, styleValue);
};

YAHOO.DataIllusion.FeedbackServer.CSSStyle.prototype =
{
	styleName : null,
	styleValue: null,

	init:function(styleName, styleValue)
	{
		this.styleName = styleName;
		this.styleValue = styleValue; 
	}
};

YAHOO.register("feedbackservercsshandler",
	YAHOO.DataIllusion.FeedbackServer.CSSHandler, 
	{version: "1.1.0", build: "0"});


